# async support is valid only on Python >= 3.6
%bcond_with async
# Enable building without docs to avoid a circular dependency between this
# and python-sphinx:
%bcond_with docs

%global with_python3 1


Name:		python-jinja2
Version:	2.10
Release:	2%{?_dist_release}
Summary:	General purpose template engine
Group:		Development/Languages
License:	BSD
URL:		http://jinja.pocoo.org/
Source0:	http://pypi.python.org/packages/source/J/Jinja2/Jinja2-%{version}.tar.gz
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root
BuildArch:	noarch
BuildRequires:	python-devel
BuildRequires:	python-rpm-macros
BuildRequires:	python-setuptools
BuildRequires:	python-markupsafe
%if %{with docs}
BuildRequires:	python-sphinx
%endif # with docs
Requires:	python-babel >= 0.8
Requires:	python-markupsafe
%if 0%{?with_python3}
BuildRequires:	python3-devel
BuildRequires:	python3-rpm-macros
BuildRequires:	python3-setuptools
BuildRequires:	python3-markupsafe
# babel isn't py3k ready yet, and is only a weak dependency
#Requires:	 python3-babel >= 0.8
%endif # with_python3

BuildRequires:	python-setuptools

Vendor: Project Vine
Distribution: Vine Linux

%description
Jinja2 is a template engine written in pure Python.  It provides a
Django inspired non-XML syntax but supports inline expressions and an
optional sandboxed environment.

If you have any exposure to other text-based template languages, such
as Smarty or Django, you should feel right at home with Jinja2. It's
both designer and developer friendly by sticking to Python's
principles and adding functionality useful for templating
environments.


%if 0%{?with_python3}
%package -n python3-jinja2
Summary:	General purpose template engine
Group:		Development/Languages
Requires:	python3-markupsafe

%description -n python3-jinja2
Jinja2 is a template engine written in pure Python.  It provides a
Django inspired non-XML syntax but supports inline expressions and an
optional sandboxed environment.

If you have any exposure to other text-based template languages, such
as Smarty or Django, you should feel right at home with Jinja2. It's
both designer and developer friendly by sticking to Python's
principles and adding functionality useful for templating
environments.
%endif # with_python3


%prep
%setup -qc -n Jinja2-%{version}
# cleanup
find Jinja2-%{version} -name '*.pyo' -o -name '*.pyc' -delete

# fix EOL
sed -i 's|\r$||g' Jinja2-%{version}/LICENSE

mv Jinja2-%{version} python3
cp -av python3 python2

%build
pushd python2
%py_build
%if %{with docs}
make -C docs html PYTHONPATH=$(pwd) SPHINXBUILD=sphinx-build-2
%endif # with docs
popd

%if 0%{?with_python3}
pushd python3
%py3_build
%if %{with docs}
make -C docs html PYTHONPATH=$(pwd) SPHINXBUILD=sphinx-build-3
%endif # with docs
popd
%endif # with_python3


%install
%__rm -rf %{buildroot}
pushd python2
%py_install

# remove hidden file
rm -rf docs/_build/html/.buildinfo

# these files are valid only on Python 3.6+
rm %{buildroot}%{python_sitelib}/jinja2/asyncsupport.py
rm %{buildroot}%{python_sitelib}/jinja2/asyncfilters.py
popd

%if %{with python3}
pushd python3
%py3_install

# remove hidden file
rm -rf docs/_build/html/.buildinfo

%if ! %{with async}
# these files are valid only on Python 3.6+
rm %{buildroot}%{python3_sitelib}/jinja2/asyncsupport.py
rm %{buildroot}%{python3_sitelib}/jinja2/asyncfilters.py
%endif # ! with async
popd
%endif # with python3

%clean
%__rm -rf %{buildroot}

%files
%defattr(-,root,root,-)
%doc python2/AUTHORS
%doc python2/CHANGES.rst
%doc python2/ext
%doc python2/examples
%license python2/LICENSE
%if %{with docs}
%doc python2/docs/_build/html
%endif
%{python_sitelib}/jinja2
%{python_sitelib}/Jinja2-%{version}-py*.egg-info

%if 0%{?with_python3}
%files -n python3-jinja2
%defattr(-,root,root,-)
%doc python3/AUTHORS
%doc python3/CHANGES.rst
%doc python3/ext
%doc python3/examples
%license python2/LICENSE
%if %{with docs}
%doc python3/docs/_build/html
%endif # with docs
%{python3_sitelib}/jinja2
%{python3_sitelib}/Jinja2-%{version}-py*.egg-info
%endif # with_python3


%changelog
* Wed Nov 28 2018 Tomohiro "Tomo-p" KATO <tomop@teamgedoh.net> 2.10-2
- fixed R:.

* Tue Nov 27 2018 Tomohiro "Tomo-p" KATO <tomop@teamgedoh.net> 2.10-1
- new upstream release.
- added python3 support.

* Sun Jan 12 2014 Yoji TOYODA <bsyamato@sea.plala.or.jp> 2.5.5-3
- rebuild with VineSeed environment

* Sun Feb 19 2012 Yoji TOYODA <bsyamato@sea.plala.or.jp> 2.5.5-2
- rebuild with python-2.7.2

* Thu May  5 2011 IWAI, Masaharu <iwai@alib.jp> 2.5.5-1
- new upstream release
- add Project and Distribution tags

* Mon Aug 23 2010 Munehiro Yamamoto <munepi@vinelinux.org> - 2.5.2-1
- initial build based on Fedora development

* Thu Aug 19 2010 Thomas Moschny <thomas.moschny@gmx.de> - 2.5.2-1
- Update to upstream version 2.5.2.
- Package depends on python-markupsafe and is noarch now.

* Thu Jul 22 2010 David Malcolm <dmalcolm@redhat.com> - 2.5-4
- add explicit build-requirement on python-setuptools
- fix doc disablement for python3 subpackage

* Thu Jul 22 2010 David Malcolm <dmalcolm@redhat.com> - 2.5-3
- support disabling documentation in the build to break a circular build-time
dependency with python-sphinx; disable docs for now

* Thu Jul 22 2010 David Malcolm <dmalcolm@redhat.com> - 2.5-2
- Rebuilt for https://fedoraproject.org/wiki/Features/Python_2.7/MassRebuild

* Tue Jul 13 2010 Thomas Moschny <thomas.moschny@gmx.de> - 2.5-1
- Update to upstream version 2.5.
- Create python3 subpackage. 
- Minor specfile fixes.
- Add examples directory.
- Thanks to Gareth Armstrong for additional hints.

* Wed Apr 21 2010 Thomas Moschny <thomas.moschny@gmx.de> - 2.4.1-1
- Update to 2.4.1.

* Tue Apr 13 2010 Thomas Moschny <thomas.moschny@gmx.de> - 2.4-1
- Update to 2.4.

* Tue Feb 23 2010 Thomas Moschny <thomas.moschny@gmx.de> - 2.3.1-1
- Update to 2.3.1.
- Docs are built using Sphinx now.
- Run the testsuite.

* Sat Sep 19 2009 Thomas Moschny <thomas.moschny@gmx.de> - 2.2.1-1
- Update to 2.2.1, mainly a bugfix release.
- Remove patch no longer needed.
- Remove conditional for FC-8.
- Compilation of speedup module has to be explicitly requested now.

* Sun Jul 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Thu Feb 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1.1-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Sat Jan 10 2009 Thomas Moschny <thomas.moschny@gmx.de> - 2.1.1-1
- Update to 2.1.1 (bugfix release).

* Thu Dec 18 2008 Thomas Moschny <thomas.moschny@gmx.de> - 2.1-1
- Update to 2.1, which fixes a number of bugs.
  See http://jinja.pocoo.org/2/documentation/changelog#version-2-1.

* Sat Nov 29 2008 Ignacio Vazquez-Abrams <ivazqueznet+rpm@gmail.com> - 2.0-3
- Rebuild for Python 2.6

* Tue Jul 22 2008 Thomas Moschny <thomas.moschny@gmx.de> - 2.0-2
- Use rpm buildroot macro instead of RPM_BUILD_ROOT.

* Sun Jul 20 2008 Thomas Moschny <thomas.moschny@gmx.de> - 2.0-1
- Upstream released 2.0.

* Sun Jun 29 2008 Thomas Moschny <thomas.moschny@gmx.de> - 2.0-0.1.rc1
- Modified specfile from the existing python-jinja package.
